//  .. 
//    crypt32.lib
//   Visual Studio 7.0

#include <stdio.h>
#include <stdlib.h> // exit(1);
#include <wtypes.h>
#include <wincrypt.h>

#define MY_TYPE  (PKCS_7_ASN_ENCODING | X509_ASN_ENCODING)

//   
#define CERT_STORE_NAME  L"MY"

//      
#define SIGNER_NAME  L"EVGENY"

//  hex    
int hex2bin(const char* hexsign, BYTE* pbarray);

void HandleError(char *s);

int main(int argc, char* argv[])
{
//--------------------------------------------------------------------
// ,   

	BYTE* pbMessage = (BYTE*)"  -  N1";
	DWORD cbMessage = (DWORD)strlen((char*) pbMessage)+1;    

//      hex 
	const char* hexsign = 
		"667387adc8449acabc3378df8225136f2e1cfca612d202e58476bde34b6251cd"
		"2ca3c9be22fbce27e504d69920266c8c4611f2d82384b7115894c1128b239e05"
		"9a5d83af8b91ec82944fb4bf75d4d5a071afdd426b3670387f128d36250fb0ba"
		"493ca49747133f62d982e5c5b9d08b5e9e5c70db3d532950350803a8e937df71";

//         

	DWORD cbArray = (DWORD)strlen(hexsign)/2 ;
	BYTE* pbArray = new BYTE[cbArray];

//      
	if (hex2bin(hexsign, pbArray))
	{
		printf(" hex2bin.\n");
		exit(1);
	}

//   

	HCERTSTORE hStoreHandle;   	

	if ( !( hStoreHandle = CertOpenStore(
		CERT_STORE_PROV_SYSTEM,
		0,
		NULL,
		CERT_SYSTEM_STORE_CURRENT_USER,
		//CERT_SYSTEM_STORE_LOCAL_MACHINE,
		CERT_STORE_NAME)))
	{
		HandleError("   MY.");
	}

//     

	PCCERT_CONTEXT pSignerCert; 

	if(pSignerCert = CertFindCertificateInStore(
		hStoreHandle,
		MY_TYPE,
		0,
		CERT_FIND_SUBJECT_STR,
		SIGNER_NAME,
		NULL))
	{
		printf(" .\n");
	}
	else
	{
		HandleError( "  .");
	}


//   CryptAcquireCertificatePrivateKey    CSP
//         CryptoAPI

	HCRYPTPROV hProv;
	DWORD dwKeySpec; 
	BOOL fCallerFreeProv; 

	if(CryptAcquireCertificatePrivateKey(
		pSignerCert, 
		CRYPT_ACQUIRE_COMPARE_KEY_FLAG, 
		NULL, 
		&hProv, 
		&dwKeySpec, 
		&fCallerFreeProv 
	))
	{
		printf("CryptAcquireCertificatePrivateKey  !\n");
	}
	else
	{
	    HandleError("  CryptAcquireCertificatePrivateKey.\n");
	}

	DWORD dwUserNameLen = 100;    
	CHAR szUserName[100];
	
	if(CryptGetProvParam(
		hProv,					//   CSP
		PP_NAME,				//     CSP
		(BYTE *)szUserName,     //   ,   CSP
		&dwUserNameLen,         //   
		0)) 
	{
		printf(" CSP: %s\n",szUserName);
	}
	else
	{
		HandleError(" CryptGetProvParam.\n");
	}

	if(CryptGetProvParam(
		hProv,					//   CSP
		PP_CONTAINER,           //     key container  
		(BYTE *)szUserName,     //   ,   key container  
		&dwUserNameLen,         //  
		0)) 
	{
		printf(" key container: %s\n",szUserName);
	}
	else
	{
		HandleError(" CryptGetProvParam.\n");
	}

//  public key   
	HCRYPTKEY hPubKey;

	if(CryptImportPublicKeyInfo(
		hProv, 
		MY_TYPE, 
		&(pSignerCert->pCertInfo->SubjectPublicKeyInfo),
		&hPubKey
	))
	{
		printf(" public key.\n");
	}
	else
	{
		HandleError(" CryptAcquireContext.");
	}

//   hash 
	HCRYPTHASH hHash;

	if(CryptCreateHash(
		hProv, 
		CALG_MD5, 
		0, 
		0, 
		&hHash)) 
	{
		printf("Hash  .\n");
	}
	else
	{
		HandleError(" CryptCreateHash.");
	}
//--------------------------------------------------------------------
// Compute the cryptographic hash of the buffer.
//  hash   
	if(CryptHashData(
		hHash, 
		pbMessage, 
		cbMessage, 
		0)) 
	{
		printf("Hash  : ");
	}
	else
	{
		HandleError("  CryptHashData.");
	}

//   
	DWORD	dwHashLen = 0;

	if(!CryptGetHashParam(
		hHash, 
		HP_HASHVAL, 
		NULL, 
		&dwHashLen, 
		0)) 
	{
		printf("    .\n");
		exit(1);
	}

//   
	BYTE *pbHash=new BYTE[dwHashLen];

	if(CryptGetHashParam(
		hHash, 
		HP_HASHVAL, 
		pbHash, 
		&dwHashLen, 
		0)) 
	{
		for(DWORD i = 0 ; i < dwHashLen ; i++) 
			printf("%2.2x",pbHash[i]);
		printf("\n");
	}
	else
	{
		printf("    .\n");
		exit(1);
	}

	if(pbHash)
		delete pbHash;

//  
	if(CryptVerifySignature(
		hHash,					//   hash 
		pbArray,				//   
		cbArray,				//  
		hPubKey,				//   public key
		NULL, 
		0)) 
	{
		printf(" .\n");
	}
	else
	{
		HandleError(" CryptVerifySignature .");
	}

//  
	if(pbArray)
		delete pbArray;

//  
	if(hHash) 
		CryptDestroyHash(hHash);

	if(hPubKey) 
		CryptDestroyKey(hPubKey);

	if(hProv) 
		CryptReleaseContext(hProv, 0);

	if(pSignerCert)
		CertFreeCertificateContext(pSignerCert);

	if(CertCloseStore(
      hStoreHandle, 
      CERT_CLOSE_STORE_CHECK_FLAG))
	{
		printf(" . \n");
	}
	else
	{
		printf("!");
	}

	return 0;
}


void HandleError(char *s)
{
    printf(". \n");
    printf("%s\n",s);
    printf(" N %x.\n", GetLastError());
    printf("   . \n");
    exit(1);
} 

int hex2int(char ch) throw(...)
{
	if (ch >= '0' && ch <= '9') return ch - '0';
	if (ch >= 'a' && ch <= 'f') return ch - 'a' + 0xA;
	if (ch >= 'A' && ch <= 'F') return ch - 'A' + 0xA;
	throw 1;		
}

int hex2bin(const char* hexsign, BYTE* pbarray)
{
	try{
		while(*hexsign)
		{
			int aa = hex2int(*hexsign++); 
			int bb = hex2int(*hexsign++);
			*pbarray++ = (BYTE)(aa << 4 | bb); 
		}
	}
	catch(...) {
		return -1;
	}
	return 0;
}
